@@ -12,25 +12,33 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import os
+import shlex
 import sys
-from typing import Any, Iterable, List, Optional, Sequence, Union
+from typing import Any, Iterable, Sequence
 
 import py
 
 from nox.logger import logger
 from nox.popen import popen
 
+if sys.version_info < (3, 8):  # pragma: no cover
+    from typing_extensions import Literal
+else:  # pragma: no cover
+    from typing import Literal
+
 
 class CommandFailed(Exception):
     """Raised when an executed command returns a non-success status code."""
 
-    def __init__(self, reason: str = None) -> None:
-        super(CommandFailed, self).__init__(reason)
+    def __init__(self, reason: str | None = None) -> None:
+        super().__init__(reason)
         self.reason = reason
 
 
-def which(program: str, paths: Optional[List[str]]) -> str:
+def which(program: str, paths: list[str] | None) -> str:
     """Finds the full path to an executable."""
     full_path = None
 
@@ -38,18 +46,18 @@ def which(program: str, paths: Optional[List[str]]) -> str:
         full_path = py.path.local.sysfind(program, paths=paths)
 
     if full_path:
-        return full_path.strpath
+        return full_path.strpath  # type: ignore[no-any-return]
 
     full_path = py.path.local.sysfind(program)
 
     if full_path:
-        return full_path.strpath
+        return full_path.strpath  # type: ignore[no-any-return]
 
-    logger.error("Program {} not found.".format(program))
-    raise CommandFailed("Program {} not found".format(program))
+    logger.error(f"Program {program} not found.")
+    raise CommandFailed(f"Program {program} not found")
 
 
-def _clean_env(env: Optional[dict]) -> Optional[dict]:
+def _clean_env(env: dict[str, str] | None) -> dict[str, str] | None:
     if env is None:
         return None
 
@@ -62,24 +70,29 @@ def _clean_env(env: Optional[dict]) -> Optional[dict]:
     return clean_env
 
 
+def _shlex_join(args: Sequence[str]) -> str:
+    # shlex.join() was added in Python 3.8
+    return " ".join(shlex.quote(arg) for arg in args)
+
+
 def run(
     args: Sequence[str],
     *,
-    env: Optional[dict] = None,
+    env: dict[str, str] | None = None,
     silent: bool = False,
-    paths: Optional[List[str]] = None,
-    success_codes: Optional[Iterable[int]] = None,
+    paths: list[str] | None = None,
+    success_codes: Iterable[int] | None = None,
     log: bool = True,
-    external: bool = False,
-    **popen_kws: Any
-) -> Union[str, bool]:
+    external: Literal["error"] | bool = False,
+    **popen_kws: Any,
+) -> str | bool:
     """Run a command-line program."""
 
     if success_codes is None:
         success_codes = [0]
 
     cmd, args = args[0], args[1:]
-    full_cmd = "{} {}".format(cmd, " ".join(args))
+    full_cmd = f"{cmd} {_shlex_join(args)}"
 
     cmd_path = which(cmd, paths)
 
@@ -92,18 +105,16 @@ def run(
         if is_external_tool:
             if external == "error":
                 logger.error(
-                    "Error: {} is not installed into the virtualenv, it is located at {}. "
-                    "Pass external=True into run() to explicitly allow this.".format(
-                        cmd, cmd_path
-                    )
+                    f"Error: {cmd} is not installed into the virtualenv, it is located"
+                    f" at {cmd_path}. Pass external=True into run() to explicitly allow"
+                    " this."
                 )
                 raise CommandFailed("External program disallowed.")
             elif external is False:
                 logger.warning(
-                    "Warning: {} is not installed into the virtualenv, it is located at {}. This might cause issues! "
-                    "Pass external=True into run() to silence this message.".format(
-                        cmd, cmd_path
-                    )
+                    f"Warning: {cmd} is not installed into the virtualenv, it is"
+                    f" located at {cmd_path}. This might cause issues! Pass"
+                    " external=True into run() to silence this message."
                 )
 
     env = _clean_env(env)
@@ -114,16 +125,15 @@ def run(
         )
 
         if return_code not in success_codes:
+            suffix = ":" if silent else ""
             logger.error(
-                "Command {} failed with exit code {}{}".format(
-                    full_cmd, return_code, ":" if silent else ""
-                )
+                f"Command {full_cmd} failed with exit code {return_code}{suffix}"
             )
 
             if silent:
                 sys.stderr.write(output)
 
-            raise CommandFailed("Returned code {}".format(return_code))
+            raise CommandFailed(f"Returned code {return_code}")
 
         if output:
             logger.output(output)
