@@ -12,23 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import argparse
 import ast
-import collections.abc
 import itertools
 from collections import OrderedDict
-from typing import (
-    Any,
-    Iterable,
-    Iterator,
-    List,
-    Mapping,
-    Optional,
-    Sequence,
-    Set,
-    Tuple,
-    Union,
-)
+from typing import Any, Iterable, Iterator, Mapping, Sequence
 
 from nox._decorators import Call, Func
 from nox.sessions import Session, SessionRunner
@@ -36,7 +26,7 @@ from nox.sessions import Session, SessionRunner
 WARN_PYTHONS_IGNORED = "python_ignored"
 
 
-def _unique_list(*args: str) -> List[str]:
+def _unique_list(*args: str) -> list[str]:
     """Return a list without duplicates, while preserving order."""
     return list(OrderedDict.fromkeys(args))
 
@@ -45,7 +35,7 @@ class Manifest:
     """Session manifest.
 
     The session manifest provides the source of truth for the sequence of
-    sessions that should be run by nox.
+    sessions that should be run by Nox.
 
     It is possible for this to be mutated during execution. This allows for
     useful use cases, such as for one session to "notify" another or
@@ -55,22 +45,28 @@ class Manifest:
         session_functions (Mapping[str, function]): The registry of discovered
             session functions.
         global_config (.nox.main.GlobalConfig): The global configuration.
+        module_docstring (Optional[str]): The user noxfile.py docstring.
+            Defaults to `None`.
     """
 
     def __init__(
-        self, session_functions: Mapping[str, "Func"], global_config: argparse.Namespace
+        self,
+        session_functions: Mapping[str, Func],
+        global_config: argparse.Namespace,
+        module_docstring: str | None = None,
     ) -> None:
-        self._all_sessions = []  # type: List[SessionRunner]
-        self._queue = []  # type: List[SessionRunner]
-        self._consumed = []  # type: List[SessionRunner]
-        self._config = global_config  # type: argparse.Namespace
+        self._all_sessions: list[SessionRunner] = []
+        self._queue: list[SessionRunner] = []
+        self._consumed: list[SessionRunner] = []
+        self._config: argparse.Namespace = global_config
+        self.module_docstring: str | None = module_docstring
 
         # Create the sessions based on the provided session functions.
         for name, func in session_functions.items():
             for session in self.make_session(name, func):
                 self.add_session(session)
 
-    def __contains__(self, needle: Union[str, SessionRunner]) -> bool:
+    def __contains__(self, needle: str | SessionRunner) -> bool:
         if needle in self._queue or needle in self._consumed:
             return True
         for session in self._queue + self._consumed:
@@ -78,7 +74,7 @@ class Manifest:
                 return True
         return False
 
-    def __iter__(self) -> "Manifest":
+    def __iter__(self) -> Manifest:
         return self
 
     def __getitem__(self, key: str) -> SessionRunner:
@@ -102,7 +98,7 @@ class Manifest:
     def __len__(self) -> int:
         return len(self._queue) + len(self._consumed)
 
-    def list_all_sessions(self) -> Iterator[Tuple[SessionRunner, bool]]:
+    def list_all_sessions(self) -> Iterator[tuple[SessionRunner, bool]]:
         """Yields all sessions and whether or not they're selected."""
         for session in self._all_sessions:
             yield session, session in self._queue
@@ -156,7 +152,7 @@ class Manifest:
             if _normalize_arg(session_name) not in all_sessions
         ]
         if missing_sessions:
-            raise KeyError("Sessions not found: {}".format(", ".join(missing_sessions)))
+            raise KeyError(f"Sessions not found: {', '.join(missing_sessions)}")
 
     def filter_by_python_interpreter(self, specified_pythons: Sequence[str]) -> None:
         """Filter sessions in the queue based on the user-specified
@@ -176,12 +172,23 @@ class Manifest:
                 session names are checked against.
         """
         self._queue = [
-            x for x in self._queue if keyword_match(keywords, x.signatures + [x.name])
+            x
+            for x in self._queue
+            if keyword_match(keywords, x.signatures + x.tags + [x.name])
         ]
 
+    def filter_by_tags(self, tags: list[str]) -> None:
+        """Filter sessions by their tags.
+
+        Args:
+            tags (list[str]): A list of tags which session names
+                are checked against.
+        """
+        self._queue = [x for x in self._queue if set(x.tags).intersection(tags)]
+
     def make_session(
-        self, name: str, func: "Func", multi: bool = False
-    ) -> List[SessionRunner]:
+        self, name: str, func: Func, multi: bool = False
+    ) -> list[SessionRunner]:
         """Create a session object from the session function.
 
         Args:
@@ -196,7 +203,7 @@ class Manifest:
         """
         sessions = []
 
-        # if backend is none we wont parametrize the pythons
+        # If the backend is "none", we won't parametrize `python`.
         backend = (
             self._config.force_venv_backend
             or func.venv_backend
@@ -211,7 +218,7 @@ class Manifest:
         if self._config.extra_pythons:
             # If extra python is provided, expand the func.python list to
             # include additional python interpreters
-            extra_pythons = self._config.extra_pythons  # type: List[str]
+            extra_pythons: list[str] = self._config.extra_pythons
             if isinstance(func.python, (list, tuple, set)):
                 func.python = _unique_list(*func.python, *extra_pythons)
             elif not multi and func.python:
@@ -235,30 +242,28 @@ class Manifest:
             return sessions
 
         # Simple case: If this function is not parametrized, then make
-        # a simple session
+        # a simple session.
         if not hasattr(func, "parametrize"):
             long_names = []
             if not multi:
                 long_names.append(name)
             if func.python:
-                long_names.append("{}-{}".format(name, func.python))
+                long_names.append(f"{name}-{func.python}")
 
             return [SessionRunner(name, long_names, func, self._config, self)]
 
         # Since this function is parametrized, we need to add a distinct
         # session for each permutation.
-        parametrize = func.parametrize  # type: ignore
+        parametrize = func.parametrize  # type: ignore[attr-defined]
         calls = Call.generate_calls(func, parametrize)
         for call in calls:
             long_names = []
             if not multi:
-                long_names.append("{}{}".format(name, call.session_signature))
+                long_names.append(f"{name}{call.session_signature}")
             if func.python:
-                long_names.append(
-                    "{}-{}{}".format(name, func.python, call.session_signature)
-                )
+                long_names.append(f"{name}-{func.python}{call.session_signature}")
                 # Ensure that specifying session-python will run all parameterizations.
-                long_names.append("{}-{}".format(name, func.python))
+                long_names.append(f"{name}-{func.python}")
 
             sessions.append(SessionRunner(name, long_names, call, self._config, self))
 
@@ -276,7 +281,7 @@ class Manifest:
         return self.__next__()
 
     def notify(
-        self, session: Union[str, SessionRunner], posargs: Optional[List[str]] = None
+        self, session: str | SessionRunner, posargs: list[str] | None = None
     ) -> bool:
         """Enqueue the specified session in the queue.
 
@@ -312,10 +317,10 @@ class Manifest:
                 return True
 
         # The session was not found in the list of sessions.
-        raise ValueError("Session {} not found.".format(session))
+        raise ValueError(f"Session {session} not found.")
 
 
-class KeywordLocals(collections.abc.Mapping):
+class KeywordLocals(Mapping[str, bool]):
     """Eval locals using keywords.
 
     When looking up a local variable the variable name is compared against
@@ -324,7 +329,7 @@ class KeywordLocals(collections.abc.Mapping):
     returns False.
     """
 
-    def __init__(self, keywords: Set[str]) -> None:
+    def __init__(self, keywords: set[str]) -> None:
         self._keywords = keywords
 
     def __getitem__(self, variable_name: str) -> bool:
@@ -363,7 +368,7 @@ def _normalized_session_match(session_name: str, session: SessionRunner) -> bool
     return False
 
 
-def _normalize_arg(arg: str) -> Union[str]:
+def _normalize_arg(arg: str) -> str:
     """Normalize arg for comparison."""
     try:
         return str(ast.dump(ast.parse(arg)))
