@@ -12,16 +12,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import os
 import platform
 import re
 import shutil
 import sys
 from socket import gethostbyname
-from typing import Any, List, Mapping, Optional, Tuple, Union
+from typing import Any, Mapping
 
 import py
 
+import nox
 import nox.command
 from nox.logger import logger
 
@@ -38,7 +41,7 @@ _ENABLE_STALENESS_CHECK = "NOX_ENABLE_STALENESS_CHECK" in os.environ
 
 class InterpreterNotFound(OSError):
     def __init__(self, interpreter: str) -> None:
-        super().__init__("Python interpreter {} not found".format(interpreter))
+        super().__init__(f"Python interpreter {interpreter} not found")
         self.interpreter = interpreter
 
 
@@ -51,9 +54,11 @@ class ProcessEnv:
     is_sandboxed = False
 
     # Special programs that aren't included in the environment.
-    allowed_globals = ()  # type: _typing.ClassVar[Tuple[Any, ...]]
+    allowed_globals: _typing.ClassVar[tuple[Any, ...]] = ()
 
-    def __init__(self, bin_paths: None = None, env: Mapping[str, str] = None) -> None:
+    def __init__(
+        self, bin_paths: None = None, env: Mapping[str, str] | None = None
+    ) -> None:
         self._bin_paths = bin_paths
         self.env = os.environ.copy()
         self._reused = False
@@ -70,7 +75,7 @@ class ProcessEnv:
             )
 
     @property
-    def bin_paths(self) -> Optional[List[str]]:
+    def bin_paths(self) -> list[str] | None:
         return self._bin_paths
 
     @property
@@ -85,12 +90,12 @@ class ProcessEnv:
         raise NotImplementedError("ProcessEnv.create should be overwritten in subclass")
 
 
-def locate_via_py(version: str) -> Optional[str]:
+def locate_via_py(version: str) -> str | None:
     """Find the Python executable using the Windows Launcher.
 
     This is based on :pep:397 which details that executing
-    ``py.exe -{version}`` should execute python with the requested
-    version. We then make the python process print out its full
+    ``py.exe -{version}`` should execute Python with the requested
+    version. We then make the Python process print out its full
     executable path which we use as the location for the version-
     specific Python interpreter.
 
@@ -107,13 +112,13 @@ def locate_via_py(version: str) -> Optional[str]:
     py_exe = py.path.local.sysfind("py")
     if py_exe is not None:
         try:
-            return py_exe.sysexec("-" + version, "-c", script).strip()
+            return py_exe.sysexec("-" + version, "-c", script).strip()  # type: ignore[no-any-return]
         except py.process.cmdexec.Error:
             return None
     return None
 
 
-def locate_using_path_and_version(version: str) -> Optional[str]:
+def locate_using_path_and_version(version: str) -> str | None:
     """Check the PATH's python interpreter and return it if the version
     matches.
 
@@ -136,7 +141,7 @@ def locate_using_path_and_version(version: str) -> Optional[str]:
     path_python = py.path.local.sysfind("python")
     if path_python:
         try:
-            prefix = "{}.".format(version)
+            prefix = f"{version}."
             version_string = path_python.sysexec("-c", script).strip()
             if version_string.startswith(prefix):
                 return str(path_python)
@@ -147,12 +152,14 @@ def locate_using_path_and_version(version: str) -> Optional[str]:
 
 
 class PassthroughEnv(ProcessEnv):
-    """Represents the environment used to run nox itself
+    """Represents the environment used to run Nox itself
 
     For now, this class is empty but it might contain tools to grasp some
     hints about the actual env.
     """
 
+    conda_cmd = "conda"
+
     @staticmethod
     def is_offline() -> bool:
         """As of now this is only used in conda_install"""
@@ -177,24 +184,28 @@ class CondaEnv(ProcessEnv):
             If not specified, this will use the currently running Python.
         reuse_existing (Optional[bool]): Flag indicating if the conda environment
             should be reused if it already exists at ``location``.
+        conda_cmd (str): The name of the command, can be "conda" (default) or "mamba".
     """
 
     is_sandboxed = True
-    allowed_globals = ("conda",)
+    allowed_globals = ("conda", "mamba")
 
     def __init__(
         self,
         location: str,
-        interpreter: Optional[str] = None,
+        interpreter: str | None = None,
         reuse_existing: bool = False,
         venv_params: Any = None,
+        *,
+        conda_cmd: str = "conda",
     ):
         self.location_name = location
         self.location = os.path.abspath(location)
         self.interpreter = interpreter
         self.reuse_existing = reuse_existing
         self.venv_params = venv_params if venv_params else []
-        super(CondaEnv, self).__init__()
+        self.conda_cmd = conda_cmd
+        super().__init__(env={"CONDA_PREFIX": self.location})
 
     def _clean_location(self) -> bool:
         """Deletes existing conda environment"""
@@ -202,7 +213,14 @@ class CondaEnv(ProcessEnv):
             if self.reuse_existing:
                 return False
             else:
-                cmd = ["conda", "remove", "--yes", "--prefix", self.location, "--all"]
+                cmd = [
+                    self.conda_cmd,
+                    "remove",
+                    "--yes",
+                    "--prefix",
+                    self.location,
+                    "--all",
+                ]
                 nox.command.run(cmd, silent=True, log=False)
                 # Make sure that location is clean
                 try:
@@ -213,26 +231,31 @@ class CondaEnv(ProcessEnv):
         return True
 
     @property
-    def bin_paths(self) -> List[str]:
+    def bin_paths(self) -> list[str]:
         """Returns the location of the conda env's bin folder."""
-        # see https://docs.anaconda.com/anaconda/user-guide/tasks/integration/python-path/#examples
+        # see https://github.com/conda/conda/blob/f60f0f1643af04ed9a51da3dd4fa242de81e32f4/conda/activate.py#L563-L572
         if _SYSTEM == "Windows":
-            return [self.location, os.path.join(self.location, "Scripts")]
+            return [
+                self.location,
+                os.path.join(self.location, "Library", "mingw-w64", "bin"),
+                os.path.join(self.location, "Library", "usr", "bin"),
+                os.path.join(self.location, "Library", "bin"),
+                os.path.join(self.location, "Scripts"),
+                os.path.join(self.location, "bin"),
+            ]
         else:
             return [os.path.join(self.location, "bin")]
 
     def create(self) -> bool:
         """Create the conda env."""
         if not self._clean_location():
-            logger.debug(
-                "Re-using existing conda env at {}.".format(self.location_name)
-            )
+            logger.debug(f"Re-using existing conda env at {self.location_name}.")
 
             self._reused = True
 
             return False
 
-        cmd = ["conda", "create", "--yes", "--prefix", self.location]
+        cmd = [self.conda_cmd, "create", "--yes", "--prefix", self.location]
 
         cmd.extend(self.venv_params)
 
@@ -240,15 +263,13 @@ class CondaEnv(ProcessEnv):
         cmd.append("pip")
 
         if self.interpreter:
-            python_dep = "python={}".format(self.interpreter)
+            python_dep = f"python={self.interpreter}"
         else:
             python_dep = "python"
         cmd.append(python_dep)
 
-        logger.info(
-            "Creating conda env in {} with {}".format(self.location_name, python_dep)
-        )
-        nox.command.run(cmd, silent=True, log=False)
+        logger.info(f"Creating conda env in {self.location_name} with {python_dep}")
+        nox.command.run(cmd, silent=True, log=nox.options.verbose or False)
 
         return True
 
@@ -266,7 +287,7 @@ class CondaEnv(ProcessEnv):
             # DNS resolution to detect situation (1) or (2).
             host = gethostbyname("repo.anaconda.com")
             return host is None
-        except:  # pragma: no cover # noqa E722
+        except BaseException:  # pragma: no cover
             return True
 
 
@@ -295,7 +316,7 @@ class VirtualEnv(ProcessEnv):
     def __init__(
         self,
         location: str,
-        interpreter: Optional[str] = None,
+        interpreter: str | None = None,
         reuse_existing: bool = False,
         *,
         venv: bool = False,
@@ -304,11 +325,11 @@ class VirtualEnv(ProcessEnv):
         self.location_name = location
         self.location = os.path.abspath(location)
         self.interpreter = interpreter
-        self._resolved = None  # type: Union[None, str, InterpreterNotFound]
+        self._resolved: None | str | InterpreterNotFound = None
         self.reuse_existing = reuse_existing
         self.venv_or_virtualenv = "venv" if venv else "virtualenv"
         self.venv_params = venv_params if venv_params else []
-        super(VirtualEnv, self).__init__(env={"VIRTUAL_ENV": self.location})
+        super().__init__(env={"VIRTUAL_ENV": self.location})
 
     def _clean_location(self) -> bool:
         """Deletes any existing virtual environment"""
@@ -360,7 +381,7 @@ class VirtualEnv(ProcessEnv):
 
         return original == created
 
-    def _read_base_prefix_from_pyvenv_cfg(self) -> Optional[str]:
+    def _read_base_prefix_from_pyvenv_cfg(self) -> str | None:
         """Return the base-prefix entry from pyvenv.cfg, if present."""
         path = os.path.join(self.location, "pyvenv.cfg")
         if os.path.isfile(path):
@@ -400,7 +421,7 @@ class VirtualEnv(ProcessEnv):
         match = re.match(r"^(?P<xy_ver>\d(\.\d+)?)(\.\d+)?$", self.interpreter)
         if match:
             xy_version = match.group("xy_ver")
-            cleaned_interpreter = "python{}".format(xy_version)
+            cleaned_interpreter = f"python{xy_version}"
 
         # If the cleaned interpreter is on the PATH, go ahead and return it.
         if py.path.local.sysfind(cleaned_interpreter):
@@ -436,7 +457,7 @@ class VirtualEnv(ProcessEnv):
         raise self._resolved
 
     @property
-    def bin_paths(self) -> List[str]:
+    def bin_paths(self) -> list[str]:
         """Returns the location of the virtualenv's bin folder."""
         if _SYSTEM == "Windows":
             return [os.path.join(self.location, "Scripts")]
@@ -447,9 +468,7 @@ class VirtualEnv(ProcessEnv):
         """Create the virtualenv or venv."""
         if not self._clean_location():
             logger.debug(
-                "Re-using existing virtual environment at {}.".format(
-                    self.location_name
-                )
+                f"Re-using existing virtual environment at {self.location_name}."
             )
 
             self._reused = True
@@ -464,13 +483,12 @@ class VirtualEnv(ProcessEnv):
             cmd = [self._resolved_interpreter, "-m", "venv", self.location]
         cmd.extend(self.venv_params)
 
+        resolved_interpreter_name = os.path.basename(self._resolved_interpreter)
+
         logger.info(
-            "Creating virtual environment ({}) using {} in {}".format(
-                self.venv_or_virtualenv,
-                os.path.basename(self._resolved_interpreter),
-                self.location_name,
-            )
+            f"Creating virtual environment ({self.venv_or_virtualenv}) using"
+            f" {resolved_interpreter_name} in {self.location_name}"
         )
-        nox.command.run(cmd, silent=True, log=False)
+        nox.command.run(cmd, silent=True, log=nox.options.verbose or False)
 
         return True
